class Semantic {
  constructor(matcher, action) {
    this.matcher = matcher;
    this.action = action;
  }

  toString() {
    return `Semantic(${this.matcher}, ${this.action})`;
  }

  matches(global, context) {
    return this.matcher({global, context})
  }

  apply(global, context) {
    const context_prime = Object.assign({}, context);
    this.action({global, context: context_prime});
    return context_prime;
  }
}

class Semantics {
  constructor(semantics) {
    this.semantics = semantics
    this.logs = []
  };

  applyToContext(global, context) {
    let context_prime = {}

    if (! (context instanceof Array || context instanceof Object) ){
      return context;
    }

    for ( const key in context ) {
      context_prime[key] = this.apply(global, context[key])
    }
    for (const isemantic in this.semantics) {
      const semantic = this.semantics[isemantic]
      if (semantic.matches(global, context)) {
        context_prime = semantic.apply(global, context);
        this.logs.push(`Semantics: applied ${semantic.toString()}\n to\n ${JSON.stringify(context)}\n result is ${JSON.stringify(context_prime)}`)
        break
      }
    }
    return context_prime
  }

  applyToContexts(global, contexts) {
    const contexts_prime = [];
    contexts.forEach( (context) => {
      contexts_prime.push(this.applyToContext(global, context));
    });
    return contexts_prime;
  }

  apply(global, value) {
    if ( Array.isArray(value) ) {
      return this.applyToContexts(global, value)
    } else if ( value instanceof Object) {
      return this.applyToContext(global, value)
    } else {
      return value
    }
  }
}

module.exports = { Semantic, Semantics };
